package brookes.joelmarty.semantic.bookclub.webapp;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.github.napalm.Napalm;
import com.github.napalm.freemarker.NapalmFreeMarker;

/**
 * Main entry point for the application.<p>
 * Napalm does not rely on an application server to run and embeds its own web server.
 * 
 * This class is annotated with:
 * <ul>
 * 	<li> '@Service' from Spring. This annotation declares this class as a service-layer component
 * 	<li> '@Path' with argument "/" to mark this class as the provider service for the root controller.
 * 	<br/>
 * 		Napalm needs at least one root service to offer automatic static
 * 		resources under '/static' context.
 * </ul>
 * 
 * @author joel
 *
 */
@Service @Path("/")
public class App 
{
	/**
	 * The NapalmFreemarker field that provides freemarker template functionalities.<p>
	 * Injected through spring's autowiring.
	 */
	@Autowired
	private NapalmFreeMarker fm;
	
	private static final Logger LOG = LoggerFactory.getLogger(App.class);
	
	/**
	 * Method that returns the home page for the application
	 * Uses '@GET' annotation from JAX-RS to make it respond to GET requests
	 * @return String that contains the generated html code.
	 */
	@GET
	@Produces("text/html")
	public Response getIndex(){
		return Response.ok(fm.render("index.ftl")).build();
	}
	
	/**
	 * Main entry point for the application
	 * @param args unused
	 */
    public static void main( String[] args )
    {
    	LOG.info("Fire Ze Missiles!");
        Napalm.run(8080,App.class, NapalmFreeMarker.class);
        LOG.info("Application terminated");
    }
}
