package brookes.joelmarty.semantic.bookclub.webapp.model;

import java.util.UUID;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.vocabulary.DCTerms;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * This class represents the books.<p>
 * Implements the CrudInterface interface that provides crud operations on the model.<br/>
 * The functional id for books is the ISBN number.<br/>
 * It is annotated with '@Data' from lombok that generates boilerplate code (getters, setters, ...)
 * @author joel
 *
 */
@Data
@EqualsAndHashCode(callSuper=true)
public class Book extends AbstractModel{

	private String isbn;
	private String title;
	private String author;
	private String subject;
	private String publisher;
	private int year;
	
	public static Book fromResource(Resource resource, Model m) {
		Book b = new Book();
		
		// set id
		// there seems to be a bug in Resource.getLocalName()
		// I have to strip the id from the URI instead
		String bookUri = resource.getURI();
		String[] uriParts = bookUri.split("/");
		String maybeBookId = uriParts[uriParts.length - 1];
		String[] hashedUriParts = maybeBookId.split("#");
		String bookId = hashedUriParts[hashedUriParts.length - 1];
		
		b.setId(UUID.fromString(bookId));
		
		// we get the isbn
		Property p = m.createProperty(ModelManager.SCHEMA_URI, "hasIsbn");
		Statement isbnStatement = resource.getProperty(p);
		if (isbnStatement != null)
			b.setIsbn(isbnStatement.getString());
		else
			b.setIsbn("");
		
		// creator (author)
		Statement creatorStatement = resource.getProperty(DCTerms.creator);
		if (creatorStatement != null)
			b.setAuthor(creatorStatement.getString());
		else
			b.setAuthor("");
		
		// title
		Statement titleStatement = resource.getProperty(DCTerms.title);
		if (titleStatement != null)
			b.setTitle(titleStatement.getString());
		else
			b.setTitle("");
		
		// subject
		Statement subjectStatement = resource.getProperty(DCTerms.subject);
		if (subjectStatement != null)
			b.setSubject(subjectStatement.getString());
		else
			b.setSubject("");
		
		// publisher
		Statement publisherStatement = resource.getProperty(DCTerms.publisher);
		if(publisherStatement != null)
			b.setPublisher(publisherStatement.getString());
		else
			b.setPublisher("");
		
		// year
		Statement yearStatement = resource.getProperty(DCTerms.issued);
		if (yearStatement != null)
			b.setYear(yearStatement.getInt());
		else
			b.setYear(0);
		return b;
	}
}
