package brookes.joelmarty.semantic.bookclub.webapp.model;

import java.util.Date;
import java.util.UUID;

import lombok.Data;
import lombok.EqualsAndHashCode;

import org.apache.commons.lang.time.DateFormatUtils;

import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Resource;
import com.ibm.icu.text.SimpleDateFormat;

@Data
@EqualsAndHashCode(callSuper=true)
public class Lending extends AbstractModel {

	private Book borrowedBook;
	private Member borrowedFrom;
	private Member lendTo;
	private Date borrowedOn;
	
	public static Lending fromResource(Resource resource, Model model) throws ModelException {
		
		Lending l = new Lending();
		
		try {
			// set id
			// set id
			// there seems to be a bug in Resource.getLocalName()
			// I have to strip the id from the URI instead
			String lendingUri = resource.getURI();
			String[] uriParts = lendingUri.split("/");
			String lendingId = uriParts[uriParts.length - 1];
			l.setId(UUID.fromString(lendingId));
			
			// set borrower
			Resource borrowerResource = resource.getRequiredProperty(model.createProperty(ModelManager.SCHEMA_URI, "lendTo")).getResource();
			Member borrower = Member.fromResource(borrowerResource, model, false);
			l.setLendTo(borrower);
			
			// set lender
			Resource lenderResource = resource.getRequiredProperty(model.createProperty(ModelManager.SCHEMA_URI, "borrowedFrom")).getResource();
			Member lender = Member.fromResource(lenderResource, model, false);
			l.setBorrowedFrom(lender);
			
			// set lent book
			Resource bookResource = resource.getRequiredProperty(model.createProperty(ModelManager.SCHEMA_URI, "borrowedBook")).getResource();
			Book book = Book.fromResource(bookResource, model);
			l.setBorrowedBook(book);
			
			// set date
			String borrowedDate = resource.getRequiredProperty(model.createProperty(ModelManager.SCHEMA_URI, "borrowedOn")).getLiteral().getLexicalForm();
			Date date = new SimpleDateFormat(DateFormatUtils.ISO_DATE_FORMAT.getPattern()).parse(borrowedDate); 
			l.setBorrowedOn(date);
		} catch (Exception e) {
			throw new ModelException(e);
		}
		
		return l;
	}

}
