package brookes.joelmarty.semantic.bookclub.webapp.service;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;

import brookes.joelmarty.semantic.bookclub.webapp.model.Book;
import brookes.joelmarty.semantic.bookclub.webapp.model.ModelManager;
import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.github.napalm.freemarker.NapalmFreeMarker;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.shared.DeleteDeniedException;
import com.hp.hpl.jena.shared.JenaException;
import com.hp.hpl.jena.vocabulary.RDF;

public class AbstractModelService {
	/**
	 * Logger.
	 */
	protected static final Logger LOG = LoggerFactory.getLogger(Book.class);
	
	/**
	 * Autowired Model.
	 */
	@Autowired
	@Qualifier("model")
	protected Model model;
	
	@Autowired
	@Qualifier(ModelManager.CONCEPT_MEMBER)
	protected String memberModelUri;
	
	@Autowired
	@Qualifier(ModelManager.CONCEPT_BOOK)
	protected String bookModelUri;
	
	@Autowired
	@Qualifier(ModelManager.CONCEPT_LENDING)
	protected String lendingModelUri;
	
	/**
	 * Spring's ApplicationContext is injected to get bean properties exposed in ModelManager class.
	 */
	@Autowired
	protected ApplicationContext ctx;
	
	@Autowired
	protected NapalmFreeMarker fm;
	
	/**
	 * Deletes a resource from the model.
	 * @param id The id of the resource to delete.
	 * @throws ModelException
	 */
	public void deleteMdl(String id) throws ModelException {
		try {
			Resource r = model.getResource(ModelManager.MODEL_URI + id);
			List<Statement> sti = r.listProperties().toList();
			
			for (Statement statement : sti) {
				statement.remove();
			}
		} catch (DeleteDeniedException e) {
			LOG.error("Access denied while deleting item #" + id.toString());
			throw new ModelException(e);
		} catch (JenaException e) {
			LOG.error("Error while deleting item # "+ id.toString());
			throw new ModelException(e);
		} catch (BeansException e) {
			LOG.error("Spring exception while deleting item #" + id.toString());
			throw new ModelException(e);
		}
	}
	
	protected Resource get(String id) throws Exception {
		Resource r = null;

		try {
			r = model.getResource(ModelManager.MODEL_URI + id);
			//r = model.getResource("http://cms.brookes.ac.uk/modules/P00775/2011/" + id);
		} catch (Exception e) {
			LOG.error(e.getClass().getSimpleName() + " was thrown while fetching item #" + id.toString());
			throw e;
		}
		
		return r;
		
	}
	
	protected static List<Resource> list(String uri, Model m) throws Exception {
		
		List<Resource> resources = null;

		try {
			resources = m.listResourcesWithProperty(RDF.type, m.getResource(uri)).toList();
		} catch (Exception e) {
			LOG.error(e.getClass().getSimpleName() + " was thrown while fetching items at " + uri);
			throw e;
		}
		
		return resources;
		
	}
}
