package brookes.joelmarty.semantic.bookclub.webapp.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

import org.springframework.stereotype.Service;

import brookes.joelmarty.semantic.bookclub.webapp.model.Book;
import brookes.joelmarty.semantic.bookclub.webapp.model.Lending;
import brookes.joelmarty.semantic.bookclub.webapp.model.Member;
import brookes.joelmarty.semantic.bookclub.webapp.model.ModelManager;
import brookes.joelmarty.semantic.bookclub.webapp.utils.ModelException;

import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.sparql.vocabulary.FOAF;

@Service
@Path("/members")
public class MemberService extends AbstractModelService {
	
	@GET
	@Produces("text/html")
	public Response list() {
		
		List<Member> members = new ArrayList<Member>();
		
		try {
			List<Resource> membersResources = AbstractModelService.list(memberModelUri, model);
			
			for (Resource resource : membersResources) {
				members.add(Member.fromResource(resource, model, false));
			}
		} catch (Exception e) {
			LOG.error("Error while fetching books");
			return Response.serverError().build();
		}
		
		return Response.ok(fm.render("members.ftl", "members", members)).build();
	}
	
	@POST
	@Path("add")
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public Response add(MultivaluedMap<String, String> form) {
		
		Member m = new Member();
		
		try {
			//String memberModelUri = ctx.getBean(ModelManager.CONCEPT_MEMBER, String.class);
			
			Resource memberResource = model.getResource(memberModelUri);
			
			Resource r = model.createResource(m.getUri(), memberResource);
			
			// adds email address
			r.addLiteral(FOAF.mbox, form.getFirst("email"));
			
			// adds name
			r.addLiteral(FOAF.name, form.getFirst("name"));
			
			// adds books
			List<String> booksIds = form.get("books");
			if(booksIds != null) {
				for (String b : booksIds) {
					Resource book = get(b);
					r.addProperty(model.createProperty(ModelManager.SCHEMA_URI, "ownBooks"), book);
				}
			}
			
			// add acquaintances
			List<String> acquaintancesIds = form.get("acquaintances");
			if (acquaintancesIds != null) {
				for (String a : acquaintancesIds) {
					Resource acquaintance = get(a);
					r.addProperty(FOAF.knows, acquaintance);
				}
			}
			
			model.commit();
		} catch (Exception e) {
			LOG.error("error while adding member #" + m.getId().toString());
			try {
				deleteMdl(m.getId().toString());
			} catch (ModelException me) {
				throw new WebApplicationException(Response.serverError().entity("failed in an attempt to restore model").build());
			}
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok().build();
	}
	
	@POST
	@Path("edit/{id}")
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public Response edit(@PathParam("id") String id, MultivaluedMap<String, String> form) {
		
		try {
			Resource memberResource = get(id);
			
			// updating email address
			memberResource.removeAll(FOAF.mbox);
			memberResource.addLiteral(FOAF.mbox, form.getFirst("email"));
			
			// update name
			memberResource.removeAll(FOAF.name);
			memberResource.addLiteral(FOAF.name, form.getFirst("name"));
			
			// update books
			Property ownBookProperty = model.createProperty(ModelManager.SCHEMA_URI, "ownBook");
			memberResource.removeAll(ownBookProperty);
			List<String> booksIds = form.get("books");
			if(booksIds != null) {
				for (String b : booksIds) {
					Resource book = get(b);
					memberResource.addProperty(ownBookProperty, book);
				}
			}
			
			// update acquaintances
			memberResource.removeAll(FOAF.knows);
			
			List<String> acquaintancesIds = form.get("acquaintances");
			if (acquaintancesIds != null) {
				for (String a : acquaintancesIds) {
					Resource acquaintance = get(a);
					memberResource.addProperty(FOAF.knows, acquaintance);
				}
			}
			
			model.commit();
			
		} catch (Exception e) {
			LOG.error("error while editing member #" + id);
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		return Response.ok().status(301).build();
	}
	
	@GET
	@Path("add")
	@Produces("text/html")
	public Response showAdd() {
		List<Member> members = new ArrayList<Member>();
		List<Book> books = new ArrayList<Book>();
		
		try {
			List<Resource> membersResources = AbstractModelService.list(memberModelUri, model);
			
			for (Resource resource : membersResources) {
				members.add(Member.fromResource(resource, model, false));
			}
			
			List<Resource> bookResources = AbstractModelService.list(bookModelUri, model);
			for (Resource resource : bookResources) {
				books.add(Book.fromResource(resource, model));
			}
		} catch (Exception e) {
			LOG.error("Error while fetching books");
			return Response.serverError().entity(e.getStackTrace()).build();
		}
		return Response.ok(fm.render("memberform.ftl", "operation", "add", "books", books, "acquaintances", members)).build();
	}
	
	@GET
	@Path("edit/{id}")
	@Produces("text/html")
	public Response showEdit(@PathParam("id") String memberId) {
	
		Member member = null;
		Map<String, String> members = new HashMap<String, String>();
		Map<String, String> mapForMemberAcquaintances = new HashMap<String, String>();
		
		Map<String, String> books = new HashMap<String, String>();
		Map<String, String> mapForOwnedBooks = new HashMap<String, String>();
		
		try {

			// we get the current member
			Resource memberResource = get(memberId);
			member = Member.fromResource(memberResource, model, true);
			
			// we get the list of all members
			List<Member> allMembers = new ArrayList<Member>();
			List<Resource> allMembersResources = AbstractModelService.list(memberModelUri, model);
			for (Resource mResource : allMembersResources) {
				allMembers.add(Member.fromResource(mResource, model, false));
			}
			
			
			// we get the list of acquaintances of the member
			List<Member> memberAcquaintances = member.getAcquaintances();
			
			// for each member, we add the member in the members map (id -> name)
			// if it is present in the list of this member acquaintances
			// then we set the value of the mapForMemberAcq. to true, false otherwise:
			// (id -> "true") if it is an acquaintance, false otherwise
			// so it will be selected on the form
			for (Member m : allMembers) {
				members.put(m.getId().toString(), m.getName());
				if (memberAcquaintances.contains(m))
					mapForMemberAcquaintances.put(m.getId().toString(), "true");
				else
					mapForMemberAcquaintances.put(m.getId().toString(), "false");
			}
			
			// we get the list of all books
			List<Book> allBooks = new ArrayList<Book>();
			List<Resource> allBookResources = AbstractModelService.list(bookModelUri, model);
			for (Resource bResource : allBookResources) {
				allBooks.add(Book.fromResource(bResource, model));
			}
			
			// we get the books of this member
			List<Book> memberBooks = member.getBooks();
			
			// for each book, we add the book id and name in the books map
			// if it is present in the list of this member's books
			// we set the value of the mapForOwnedBooks to true, so it will be selected on the form
			for (Book book : allBooks) {
				books.put(book.getId().toString(), book.getTitle());
				if(memberBooks.contains(book))
					mapForOwnedBooks.put(book.getId().toString(), "true");
				else
					mapForOwnedBooks.put(book.getId().toString(), "false");
			}
			
		} catch (Exception e) {
			LOG.error("Error while displaying member #" + memberId + " for edit");
			return Response.serverError().entity(e.getMessage()).build();
		}
		
		return Response.ok(fm.render("memberform.ftl",
									 "operation", "edit",
									 "member", member,
									 "members", members,
									 "map_member_acq", mapForMemberAcquaintances,
									 "books", books,
									 "map_member_books", mapForOwnedBooks)).build();
	}
	
	@GET
	@Path("{id}")
	@Produces("text/html")
	public Response show(@PathParam("id") String memberId){
		
		Member member = null;
		String queryString =
			"PREFIX bc: <http://cms.brookes.ac.uk/modules/P00775/2011/10093745/schema#>" +
			"SELECT ?lending ?book " +
			"WHERE { " +
			"?lending bc:borrowedBook ?book . " +
			"?member bc:ownBook ?book " +
			"}";
		
		List<Lending> lendings = new ArrayList<Lending>();
		Map<String, String> lendingIdToBookTitle = new HashMap<String, String>();
		
		try {
			Resource memberResource = get(memberId);
			QuerySolutionMap varMapping = new QuerySolutionMap();
			varMapping.add("member", memberResource);
			
			member = Member.fromResource(memberResource, model, true);
			
			QueryExecution query = QueryExecutionFactory.create(queryString, model, varMapping);
			ResultSet rs = query.execSelect();
			
			while(rs.hasNext()) {
				QuerySolution soln = rs.nextSolution();
				Resource bookResource = soln.getResource("book");
				Book book = Book.fromResource(bookResource, model);
				Resource lendingResource = soln.getResource("lending");
				Lending lending = Lending.fromResource(lendingResource, model);
				
				lendings.add(lending);
				lendingIdToBookTitle.put(lending.getId().toString(), book.getTitle());
			}
		} catch (Exception e) {
			LOG.error("Error while fetching member information", e);
			throw new WebApplicationException(Response.serverError().entity(e.getLocalizedMessage()).build());
		}
		
		
		return Response.ok(fm.render("member.ftl",
									 "member", member,
									 "lendings", lendings,
									 "lidtobook", lendingIdToBookTitle)).build();
	}
}
